##############################
##2013 Prefresher
##Common errors and code debugging
##Stephen Pettigrew
##Harvard University
##pettigrew@fas.harvard.edu
##############################


##### This R script shows examples of common error
##### messages in R, as well as common strategies
##### for debugging your code


#####################################################
## Common error messages



#### Common errors with character strings

# Forgetting to wrap a character string in quotation marks:
setwd(c:/users/stephen/my files/school/fall 2013/prefresher/r tutorials)
## "Error: unexpected '...' in ..."
## More generally you'll get this error if there's an extra stray letter
## or character in your code

# Proper code:
setwd("c:/users/stephen/my files/school/fall 2013/prefresher/r tutorials")






## If your code doesn't run but doesn't give you an error. Instead you
## have a + in your console.
## This generally means that you forgot to close a parentheses or quotation
## mark and R is waiting for more code because it thinks you're still inside
## the parentheses or quote

## Example:
setwd("c:/users/stephen/my files/school/fall 2013/prefresher/r tutorials"


      

      

      

## "Error ... : object '...' not found"

ncol(my.matrix.of.dataset)
# I don't have anything in my workspace called my.matrix.of.dataset

## This error can also happen when you forget to wrap a character string
## in quotation marks:

load(debuggingdata.RData)

# Correct code:
load("debuggingdata.RData")






## Object names cannot start with a number or symbol:

1st.matrix <- matrix(1:10, ncol = 2)
## "Error: unexpected symbol in "1st.matrix""







apply(dataset, 2 mean)
## "Error: unexpected symbol in "apply(dataset, 2 mean""
## This error usually means that you're missing a comma



apply(dataset, 2, mean,)
## "Error ... argument is missing, with no default"
## This error usually means you have an extra common somewhere in the code





mean(dataset[,5])
## "Error ... undefined columns/rows selected"
## This error generally means you're trying to pull out a row/column
## that doesn't actually exist in your matrix/dataframe

## It also you mean you're missing a comma inside your brackets:
mean(dataset[dataset$var1 < 0])

## Correct code:
mean(dataset[dataset$var1 < 0, 1])






## "Error: could not find function "...""
## You're trying to use a function that doesn't exist

## It could be a spelling error:
maen(dataset[,1])

## Or you might have mixed up your parentheses and brackets:
mean(dataset(1))






## "Error ... requires numeric/complex matrix/vector arguments"

## You're trying to do matrix operations on a dataframe.
## It's almost guaranteed that you'll run into this error
## in Gov2000 or Gov2001
dataset %*% t(dataset)

## Correct code:
as.matrix(dataset) %*% t(as.matrix(dataset))








########################################
## Other general strategies for when a for-loop or 
## function you wrote won't work

# For loops give you an error:

# R will store the index number in your workspace,
# so first thing to figure out is how far it got 
# through the loop before erroring out. If you index
# the loop with 'i' type 'i' into the R console.

# If the value of 'i' is not 1 (or the first iteration)
# of the loop, then it means your loop got through a few
# iterations before breaking. You'll want to figure out
# what specifically about that iteration broke the loop

# Usually though the loop breaks on the first iteration.
# To see why it broke, enter each line of code within
# the loop line by line (without running 'for(i in ...){')
# and see where it breaks.


# For functions that you wrote that won't work use a similar
# strategy. Go line by line through the function and see
# where it breaks down.

# Another trick to debug functions is to store objects in it with
# '<<-' instead of '<-'. If you only use '<-' then the only thing
# left in your workspace after the function runs will be the output
# from the final line of the function. Using '<<-' will force that
# object into your workspace, even if it's not the last line in
# the function


# Browser statements are also useful to debug loops and functions.
# Put the code 'browser()' anywhere in the code for your loop
# or function and R will pause there and let you inspect what
# the loop or function has done so far.